#ifndef CHROM_H_
#define CHROM_H_
/*

  File:           chrom.h
  Project:        Human
  Author:         Terry Furey
  Date:           11/10/00
  Description:    This class represents a chromosome
                  

*/

#include <fstream>
#include <strstream>
#include "hmm.h"

// Maximum number of bands //
const int MAX_BANDS = 100;

// Maximum number of markers //
const int MAX_MARKERS = 2500;

// Maximum characters on an input line // 
const int MAX_LINE = 1000;

// CHROM object //
class CHROM {
  
 private:
  char number[2];               // chromosome number
  long length;                  // length of chromsome in base pairs
  long pctLength;               // length of chromsome in percentage file
  long windowSize;              // size of the windows
  float slop;                   // number of slop windows for each band
  int numWindows;               // number of windows in the chromsome
  int numBands;                 // number of bands in the chromsome
  CHROM_BAND* bands[MAX_BANDS];  // list of bands in the chromsome
  int numMarkers;               // number of markers hitting the chromosome
  MARKER* markers[MAX_MARKERS];  // list of markers hitting this chromosome
  HMM predHMM;                  // HMM used to predict boundaries
  int numStates;                // number of states needed for HMM

 public:
  
  // Create a new chromsome //
  CHROM() { };
  
  // Set the band number //
  void setNumber(char* newNumber) { sprintf(number, "%s", newNumber); }
  
  // Set the length //
  void setLength(long newLength) { length = newLength; }
  
  // Set size of windows //
  void setWindowSize(int size) { windowSize = (size * 1000); numWindows = (int)(length/windowSize) + 1; }

  // Set the slop //
  void setSlop(float newSlop) { slop = newSlop; }
  
  // Set number of bands //
  void setNumBands(int newBands) { numBands = newBands; }
  
  // Read in and create the bands in this chromosome //
  void createBands(ifstream& file, int closeBands);

  // Read in and create the markers hitting this chromosome //
  int createMarkers(ifstream& file, int closeBands);

  // Read in and create set bands and markers
  void createSetBands(ifstream& file);

  // Create the HMM to determine the band positions
  void createHMM();

  // Create the viterbi trace
  void findBoundaries();

  // Determine the band predictions

  // Determine the marker predictions

  // Print the output file
  void printResults(char* label);

 private:

  // Find the beginning or ending band for a range //
  int findBand(char* band, int begin);

}; // end CHROM object

#endif
