#ifndef CHROM_BAND_H_
#define CHROM_BAND_H_
/*

  File:           chromband.h
  Project:        Human
  Author:         Terry Furey
  Date:           11/10/00
  Description:    This class represents a chromosome band
                  
*/

#include <strstream>

// Maximum number of close bands
const int MAX_CLOSE = 10;

// CHROM_BAND object //
class CHROM_BAND {

 private:
  char bandNumber[10];    // the band number, i.e. p11.1
  long pctStart;          // start base pair, based on percentages
  long pctLength;         // length of band, based on percentages
  char stain[10];         // band stain

  int numWindows;         // number of windows in this band
  int slopWindows;        // number of slop windows for the band
  char closeBands[MAX_CLOSE][10];  // list of close bands
  int numCloseBands;

  long predStart;         // predicted start location of this band
  long predLength;        // predicted length of this band

 public:
  
  // Create a new band //
  CHROM_BAND() { };

  // Return the band number //
  char* getBandNumber() { return(bandNumber); }

  // Return the percentage start //
  long getPctStart() { return(pctStart); }

  // Return the percentage length //
  long getPctLength() { return(pctLength); }

  // Return the stain for the band //
  char* getStain() { return(stain); }

  // Return the predicted start //
  long getPredStart() { return(predStart); }

  // Return the predicted length //
  long getPredLength() { return(predLength); }

  // Return number of windows in this band //
  int getNumWindows() { return(numWindows); }

  // Return number of slop windows in this band //
  int getSlopWindows() { return(slopWindows); }

  // Return number of states required for this band in the HMM //
  int getNumStates() { return(numWindows + slopWindows); }

  // Return the close band requested
  char* getCloseBand(int index) { if (index >= numCloseBands) { return(NULL); } else { return(closeBands[index]); } }

  // Set the band name //
  void setBandNumber(char* newNumber) { sprintf(bandNumber, "%s", newNumber); }

  // Set percentage start //
  void setPctStart(long newStart) { pctStart = newStart; }

  // Set percentage length //
  void setPctLength(long newLength) { pctLength = newLength; }

  // Set number of windows //
  void setNumWindows(int newNum) { numWindows = newNum; }

  // Set number of slop windows //
  void setSlopWindows(int newNum) { slopWindows = newNum; }

  // Set stain //
  void setStain(char* newStain) { sprintf(stain, "%s", newStain); }

  // Calculate number of windows //
  void calcNumWindows(long window) { numWindows = (int)(pctLength/window) + 1; }

  // Calculate number of slop windows
  void calcSlopWindows(float slop) { slopWindows = (int)(slop*numWindows); }

  // Set the predicted start //
  void setPredStart(long newStart) { predStart = newStart; }

  // Set the predicted length //
  void setPredLength(long newLength) { predLength = newLength; }

  // Add a close band //
  void addCloseBand(char* newBand) { sprintf(closeBands[numCloseBands], "%s", newBand); numCloseBands++;}

};  // end CHROM_BAND object //

#endif
