#ifndef HMM_H_
#define HMM_H_
/*

  File:           hmm.h
  Project:        Human
  Author:         Terry Furey
  Date:           10/13/00
  Description:    This class represents a hidden-Markov model (HMM)

*/

#include <fstream>
#include <strstream>
#include <string>
#include "hmmstate.h"
#include "hmmtrans.h"
#include "marker.h"

// MAX number of windows
const int MAX_HMM_BANDS = 100;
const int MAX_LINE1 = 500000;

// hmm object //
class HMM {

 public:

  HMM_STATE* states;               // The states and transitions in the HMM //
  int numStates;                   // The number of states //
  int numBands;                    // Number of bands in the chromosome
  int numWindows;                  // Number of windows in the chromsome 
  int bandLength[MAX_HMM_BANDS];
  int bandSlop[MAX_HMM_BANDS];
  int bandStart[MAX_HMM_BANDS];
  int bandEnd[MAX_HMM_BANDS];

  PROBCLASS** viterbi;            // Viterbi matrix
  int** path;                     // Viterbi path matrix
  int* maxPath;                   // Viterbi path
  PROBCLASS vitScore;             // Viterbi score

 public:

  // Create a new HMM //
  HMM() { }
 
  // Create the begin state and initialize some variables
  void createBegin(int numW, int numS);

  // Create the states for the bands
  void createBandStates(CHROM_BAND* thisBand);

  // Create the transitions for the states
  void createTransitions();

  // Run the viterbi algorithm to determine most probable path //
  int calcViterbi(MARKER* markers[], int numMarkers);

  // Create a file with the viterbi trace //
  int printViterbi(char* label);

  // Print the parameters of the HMM graph to a file //
  void printHMM(ofstream& out);

 private:

  // Free space from Viterbi matrix //
  void freeViterbi();

  // Allocate space for Viterbi matrix //
  void allocateViterbi(const long seqLength);

}; // end HMM object

#endif
