#ifndef HMM_STATE_H_
#define HMM_STATE_H_
/*

  File:           hmmstate.h
  Project:        Human
  Author:         Terry Furey
  Date:           10/13/00
  Description:    This class represents hidden-Markov model (HMM)
                  states and their transitions and emission probabilities

*/

#include <fstream>
#include "chromband.h"
#include "Prob.h"

// Pre-delcaration of HMM_TRANS for HMM_STATE //
class HMM_TRANS;

// HMM_STATE object //
class HMM_STATE {

 public:
 
  int number;                     // state number
  CHROM_BAND* stateBand;          // chromosome band associated with the state
  HMM_TRANS* transitions;         // state transitions
  int numTrans;                   // number of transitions
 
 public:
  
  // Initial creation //
  HMM_STATE();

  // Delete a HMMstate object //
  ~HMM_STATE() { } 

  // Make a state
  void makeHMMstate(CHROM_BAND* thisBand, int num);

  // Return state number //
  int getNumber() { return(number); }

  // Return band //
  CHROM_BAND* getStateBand() { return(stateBand); }

  // Return band name //
  char* getBandName() { return(stateBand->getBandNumber()); }

  // Return close bands //
  char* getCloseBand(int index) { return(stateBand->getCloseBand(index)); }

  // Add a transition from this state //
  void addTransition(PROBCLASS prob, HMM_STATE* state);

  // Return transitions from this state //
  HMM_TRANS* getTransitions() { return(transitions); }

  // Print the state parameters //
  void printHMMstate(ofstream& out);

  // Print the transition parameters for all transitions //
  void printHMMtrans(ofstream& out);

}; // end HMM_STATE

#endif
