#ifndef HMM_TRANS_H_
#define HMM_TRANS_H_
/*

  File:           hmmtrans.h
  Project:        Human
  Author:         Terry Furey
  Date:           10/13/00
  Description:    This class represents hidden-Markov model (HMM)
                  transition

*/

#include <fstream>
#include <string>
#include "Prob.h"

// Pre-delcaration of HMM_STATE for HMM_TRANS //
class HMM_STATE;

// HMMtrans object //
class HMM_TRANS {

 public:
  
  PROBCLASS probability;   // probability of this transition
  HMM_STATE* state;         // corresponding to state
  int stateNumber;         // state number of to state
  PROBCLASS count;         // number of times transition used - updating

  // public:

  // Create a new HMMtrans object //
  HMM_TRANS() { }  

  // Create a new HMMtrans object //
  void makeHMMtrans(PROBCLASS prob, HMM_STATE *st, int num) 
    { probability = prob; state = st; stateNumber = num; } 

  // Delete a HMMtransobject //
  ~HMM_TRANS() { }

  // Return the probability //
  PROBCLASS getProb() { return(probability); }

  // Return the state //
  HMM_STATE* getState() { return(state); }

  // Return the state number //
  int getStateNum() { return(stateNumber); }

  // Set the count //
  void setCount(PROBCLASS newCount) { count = newCount; }

  // Add to the current count //
  void addCount(PROBCLASS newCount) { count += newCount; }

  // Return the count //
  PROBCLASS getCount() { return(count); }

  // Update the probability //
  void updateProb(PROBCLASS newProb) { probability = newProb; }

  // Print a transition for the results file //
  void printfHMMtrans(ofstream& out) 
    { out << probability.ret_double() << endl; }

  // Print a transition to standard out for debugging //
  void printHMMtrans(ofstream& out);

}; // end HMM_TRANS object

#endif
