#ifndef MARKER_H_
#define MARKER_H_
/*

  File:           marker.h
  Project:        Human
  Author:         Terry Furey
  Date:           11/13/00
  Description:    This class represents a FISH marker
                  

*/

#include <fstream>
#include <strstream>

// Maximum number of bands a marker can span //
const int MAX_M_BANDS = 10;

// MARKER object //
class MARKER {

 private:
  char name[25];                // name of the marker
  long gpPos;                   // position on the golden path
  int  gpWindow;                // window in the golden path where this hits
  char bands[MAX_M_BANDS][10];  // bands that this marker hits
  int  numBands;                // number of bands the marker hits
  char lab[10];                 // lab marker was FISHed in 
  char predBand[10];            // band this marker is predicted to be in
  char accession[10];           // accession that this band hits

 public:
  // Create a marker //
  MARKER() { };
  
  // Return the Golden Path window //
  char* getName() { return(name); }

  // Return the Golden Path window //
  int getWindow() { return(gpWindow); }

  // Return the lab //
  char* getLab() { return(lab); }

  // Set the name of the marker //
  void setName(char* newName) { sprintf(name, "%s", newName); numBands = 0; }
  
  // Set the position on the golden path //
  void setPosition(long newPos) { gpPos = newPos; }

  // Set the window that the marker is in //
  void calcWindow(long window) { gpWindow = (int)(gpPos/window); } 
  
  // Set the lab that the marker was FISHed in //
  void setLab(char* newLab) { sprintf(lab, "%s", newLab); }
  
  // Set the predicted band //
  void setPredBand(char* newBand) { sprintf(predBand, "%s", newBand); }
  
  // Set the accession number //
  void setAccession(char* newAcc) { sprintf(accession, "%s", newAcc); }
  
  // Add a band that the marker hits //
  void addBand(char* newBand) { sprintf(bands[numBands], "%s", newBand); numBands++; }
    
  // Check if a band matches any bands this marker hits
  double bandMatch(char* checkBand, int close);

  // Print the bands to an output file //
  void printBands(ofstream& out);

}; // End MARKER object //
  
#endif
