#ifndef LOCALMEM_H
#include "localmem.h"
#endif 

#ifndef DNAUTIL_H
#include "dnautil.h"
#endif

#ifndef NT4_H
#include "nt4.h"
#endif



extern struct lm *memPool;
extern char **chromNames;
extern int chromCount;
extern boolean isFly;

/** Call this before most anything else - sets up
 * local memory pool, chromosome names, etc. */
void cdnaAliInit();

#define alloc(size) lmAlloc(memPool, size)
#define allocA(type) alloc(sizeof(type))
#define allocV(pvar) pvar = alloc(sizeof(*(pvar)))


struct cdna
    {
    struct cdna *next;
    char *name;
    int size;
    struct ali *aliList;
    boolean isUsed;
    boolean isEmbryonic;
    };

struct ali
    {
    struct ali *next;
    struct cdna *cdna;
    double score;
    char *chrom;
    char strand;
    char direction;
    char *gene;
    struct block *blockList;
    };

struct block
    {
    struct block *next;
    int nStart, nEnd, hStart, hEnd;
    int startGood, endGood, midGood;
    };

/** Return start and end of block list. */
void blockEnds(struct block *block, int *retStart, int *retEnd);

/** The minimum number of bases that we say separates genes. */
#define GENE_SPACE 1000

/** Return string in array thats same as name. */
char *findInStrings(char *name, char *strings[], int stringCount);

/** Return index of string in array. */
int ixInStrings(char *name, char *strings[], int stringCount);

/** Load up good.txt file into a cdna list. */
struct cdna *loadCdna(char *fileName);


/** Simple reference to cDNA that can put on a list. */
struct cdnaRef
    {
    struct cdnaRef *next;
    struct ali *ali;
    };

/** Returns cdnaRef associated with cdna in list, or NULL if it isn't
 * in list. */
struct cdnaRef *inRefList(struct cdnaRef *refList, struct ali *cdnaAli);

/** Make list that contains union of two reference lists. */
struct cdnaRef *addUniqRefs(struct cdnaRef *list, struct cdnaRef *newRefs);

/** Something associated with a chromosome range.  May also
 * be associated with a gene and cdna.  Sort of a sloppy,
 * multi-use structure, not all of whose fields are used by
 * everyone. */
struct feature
    {
    struct feature *next;
    char *chrom;
    int start, end;
    int startGood, endGood;
    char *gene;
    int cdnaCount;
    struct cdnaRef *cdnaRefs;
    char strand;
    };

/** Help sort features by chromosome, start, end. */
int cmpFeatures(const void *va, const void *vb);


/** Merge duplicate features. */
void collapseFeatures(struct feature *featureList);

struct feature *skipIrrelevant(struct feature *listSeg, struct feature *feat,
    int extraSpace);
/** Skip parts of listSeg that couldn't matter to feat. Assumes
 * listSeg is sorted on start. Returns first possibly relevant
 * item of listSeg. */

struct feature *sortedSearchOverlap(struct feature *listSeg, 
    struct feature *feature, int extra);
/** Assuming that listSeg is sorted on start, search through list
 * until find something that overlaps with feature, or have gone
 * past where feature could be. Returns overlapping feature or NULL. */

/** Free up genome. */
void freeGenome(struct nt4Seq ***pNt4Seq, char ***pChromNames, int chromCount);

/** Load up entire packed  genome into memory. */
void loadGenome(char *nt4Dir, struct nt4Seq ***retNt4Seq, char ***retChromNames, int *retNt4Count);

/** Create introns from cdnaList. */
struct feature *makeIntrons(struct cdna *cdnaList);

