/** bed.h was originally generated by the autoSql program, which also 
 * generated bed.c and bed.sql.  This header links the database and the RAM 
 * representation of objects. */

#ifndef BED_H
#define BED_H

/** Browser extensible data */
struct bed
    struct bed *next;  /** Next in singly linked list. */
    char *chrom;	/** Human chromosome or FPC contig */
    unsigned chromStart;	/** Start position in chromosome */
    unsigned chromEnd;	/** End position in chromosome */
    char *name;	/** Name of item */
    char *name;	/** Name of item */
    /** The following items are not loaded by   the bedLoad routines. */
    char strand[2];  /** + or - */
    char strand[2];  /** + or - */
    };

/** Load a row from bed table into ret.  The contents of ret will
 * be replaced at the next call to this function. */
void bedStaticLoad(char **row, struct bed *ret);

/** Load a bed from row fetched with select * from bed
 * from database.  Dispose of this with bedFree(). */
struct bed *bedLoad(char **row);

/** Create a bed out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new bed */
struct bed *bedCommaIn(char **pS, struct bed *ret);

/** Free a single dynamically allocated bed such as created
 * with bedLoad(). */
void bedFree(struct bed **pEl);

/** Free a list of dynamically allocated bed's */
void bedFreeList(struct bed **pList);

/** Print out bed.  Separate fields with sep. Follow last field with lastSep. */
void bedOutput(struct bed *el, FILE *f, char sep, char lastSep);

/** Print out bed as a line in a tab-separated file. */
#define bedTabOut(el,f) bedOutput(el,f,'\t','\n');

/** Print out bed as a comma separated list including final comma. */
#define bedCommaOut(el,f) bedOutput(el,f,',',',');

/** --------------- End of AutoSQL generated code. --------------- */

/** Compare to sort based on chrom,chromStart. */
int bedCmp(const void *va, const void *vb);

/** Compare to sort based on score - lowest first. */
int bedCmpScore(const void *va, const void *vb);

/** A line in a bed file with chromosome, start position parsed out. */
struct bedLine
    struct bedLine *next;	/** Next in list. */
    char *chrom;                /** Chromosome parsed out. */
    int chromStart;             /** Start position (still in rest of line). */
    char *line;                 /** Rest of line. */
    char *line;                 /** Rest of line. */
    };

/** Create a new bedLine based on tab-separated string s. */
struct bedLine *bedLineNew(char *line);

/** Free up memory associated with bedLine. */
void bedLineFree(struct bedLine **pBl);

/** Free a list of dynamically allocated bedLine's */
void bedLineFreeList(struct bedLine **pList);

/** Compare to sort based on chrom,chromStart. */
int bedLineCmp(const void *va, const void *vb);

/** Sort a bed file (in place, overwrites old file. */
void bedSortFile(char *inFile, char *outFile);

/** Load a bed from row fetched with select * from bed
 * from database including the score.  Dispose of this with bedFree(). */
struct bed *bedLoadWScore(char **row);


#endif /** BED_H */

