/** hdb - human genome browser database. */

#ifndef HDB_H
#define HDB_H

#ifndef DNAUTIL_H
#include "dnautil.h"
#endif

/** read the default settings from the config file */
void hDefaultConnect();

/** set the connection information for the database */
void hSetDbConnect(char* host, char *db, char *user, char *password);

/** Set the database name. */
void hSetDb(char *dbName);

char *hGetDb();
/** Return the current database name. */

char *hGetDbHost();
/** Return the current database host. */

char *hGetDbName();
/** Return the current database name. */

char *hGetDbUser();
/** Return the current database user. */

char *hGetDbPassword();
/** Return the current database password. */

/** Get free connection if possible. If not allocate a new one. */
struct sqlConnection *hAllocConn();

/** Put back connection for reuse. */
struct sqlConnection *hFreeConn(struct sqlConnection **pConn);

/** Return TRUE if a table exists in database. */
boolean hTableExists(char *table);

/** Return size of chromosome. */
int hChromSize(char *chromName);

/** Fetch DNA */
struct dnaSeq *hDnaFromSeq(char *seqName, int start, int end, enum dnaCase dnaCase);

/** Fetch entire chromosome into memory. */
struct dnaSeq *hLoadChrom(char *chromName);

/** Get .nib file associated with chromosome. */
void hNibForChrom(char *chromName, char retNibName[512]);

/** Get list of all chromosomes. */
struct slName *hAllChromNames();

/** Return sequence for external seq. */
struct dnaSeq *hExtSeq(char *acc);

/** Return sequence for RNA. */
struct dnaSeq *hRnaSeq(char *acc);

/** return the freeze for the database version. 
   For example: "db6" returns "Dec 12, 2000". If database
   not recognized returns NULL */
char *hFreezeFromDb(char *database);

#endif /** HDB_H */
