/*** blastParse - read in blast output into C data structure. */

#ifndef BLASTPARSE_H
#define BLASTPARSE_H

/*** All the info in a single file. */
struct blastFile
    char *fileName;			/*** Name of file this is in. */
    char *program;                      /*** Blastp, blastx, blastn, etc. */
    char *version;                      /*** Version of program. */
    char *buildDate;                    /*** Build date of program. */
    struct lineFile *lf;                /*** File blast is in. */
    struct blastQuery *queries;         /*** List of queries. */
    struct blastQuery *queries;         /*** List of queries. */
    struct blastQuery *queries;         /*** List of queries. */
    };

/*** Info on one query. */
struct blastQuery
    char *query;                        /*** Name of query sequence. */
    int queryBaseCount;                 /*** Number of bases in query. */
    char *database;                     /*** Name of database. */
    int dbSeqCount;                     /*** Number of sequences in database. */
    int dbBaseCount;                    /*** Number of bases in database. */
    struct blastGappedAli *gapped;      /*** List of gapped alignments. */
    struct blastGappedAli *gapped;      /*** List of gapped alignments. */
    struct blastGappedAli *gapped;      /*** List of gapped alignments. */
    };

/*** Info about a gapped alignment. */
struct blastGappedAli
    struct blastQuery *query;           /*** Query associated with this alignment (not owned here). */
    char *targetName;                   /*** Name of target sequence. */
    int targetSize;                     /*** Size of target sequence. */
    struct blastBlock *blocks;          /*** List of aligning blocks (no big gaps). */
    struct blastBlock *blocks;          /*** List of aligning blocks (no big gaps). */
    struct blastBlock *blocks;          /*** List of aligning blocks (no big gaps). */
    };

/*** Info about a single block of gapped alignment. */
struct blastBlock
    int bitScore;                       /*** About 2 bits per aligning nucleotide. */
    double eVal;                        /*** Expected number of alignments in database. */
    int matchCount;                     /*** Number of matching nucleotides. */
    int totalCount;                     /*** Total number of nucleotides. */
    int insertCount;                    /*** Number of inserts. */
    BYTE qStrand;                       /*** Query strand (+1 or -1) */
    BYTE tStrand;                       /*** Target strand (+1 or -1) */
    int qStart;                         /*** Query start position. */
    int tStart;                         /*** Target start position. */
    int qEnd;                           /*** Query end position. */
    int tEnd;                           /*** Target end position. */
    char *qSym;                         /*** Query letters (including '-') */
    char *tSym;                         /*** Target letters (including '-') */
    char *tSym;                         /*** Target letters (including '-') */
    char *tSym;                         /*** Target letters (including '-') */
    };

/*** Read all blast alignment in file. */
struct blastFile *blastFileReadAll(char *fileName);

/*** Open file, read and verify header. */
struct blastFile *blastFileOpenVerify(char *fileName);

/*** Read all alignments associated with next query.  Return NULL at EOF. */
struct blastQuery *blastFileNextQuery(struct blastFile *bf);

/*** Read in next gapped alignment.   Does *not* put it on bf->gapped list. 
 * Return NULL at EOF or end of query. */
struct blastGappedAli *blastFileNextGapped(struct blastFile *bf, struct blastQuery *bq);

struct blastBlock *blastFileNextBlock(struct blastFile *bf, 
	struct blastQuery *bq, struct blastGappedAli *bga);
/*** Read in next blast block.  Return NULL at EOF or end of
 * gapped alignment. */

/*** Free blast file. */
void blastFileFree(struct blastFile **pBf);

/*** Free list of blast files. */
void blastFileFreeList(struct blastFile **pList);

/*** Free single blastQuery. */
void blastQueryFree(struct blastQuery **pBq);

/*** Free list of blastQuery's. */
void blastQueryFreeList(struct blastQuery **pList);

/*** Free blastGappedAli. */
void blastGappedAliFree(struct blastGappedAli **pBga);

/*** Free blastGappedAli list. */
void blastGappedAliFreeList(struct blastGappedAli **pList);

/*** Free a single blastBlock. */
void blastBlockFree(struct blastBlock **pBb);

/*** Free a list of blastBlocks. */
void blastBlockFreeList(struct blastBlock **pList);

#endif /*** BLASTPARSE_H */

