/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** cda.h - cDNA Alignment structure.  This stores all the info except
 * the bases themselves on an cDNA alignment. */

#ifndef CDA_H
#define CDA_H

#ifndef MEMGFX_H
#include "memgfx.h"
#endif 

#ifndef FUZZYFIND_H
#include "fuzzyFind.h"
#endif

    int nStart, nEnd;           /*** Start and end position in cDNA. */
    int hStart, hEnd;           /*** Start and end position on chromosome. */
    UBYTE startGood, endGood;   /*** Number of bases matching perfectly on end */
    UBYTE midScore;             /*** 0-255  255 is perfect. */
    UBYTE midScore;             /*** 0-255  255 is perfect. */
    UBYTE midScore;             /*** 0-255  255 is perfect. */
    };

struct cdaAli
    {
    struct cdaAli *next;
    short milliScore;               /*** Score 0-1000 */
    bits16 chromIx;                 /*** On disk as just a UBYTE */
    char strand;                    /*** Strand of chromosome cDNA aligns with + or - */
    char direction;                 /*** Direction of cDNA relative to transcription + or - */
    char direction;                 /*** Direction of cDNA relative to transcription + or - */
    char direction;                 /*** Direction of cDNA relative to transcription + or - */
    int orientation;                /*** +1 or -1 depending on whether *clone* is + or - WRT chrom . */
    int orientation;                /*** +1 or -1 depending on whether *clone* is + or - WRT chrom . */
    int orientation;                /*** +1 or -1 depending on whether *clone* is + or - WRT chrom . */
                                    /*** New and perhaps not always respected. */
    struct cdaBlock *blocks;        /*** Dynamically allocated array. */
    struct cdaBlock *blocks;        /*** Dynamically allocated array. */
    struct cdaBlock *blocks;        /*** Dynamically allocated array. */
    };

/*** Returns TRUE if clone (.3/.5 pair) aligns on reverse strand. */
boolean cdaCloneIsReverse(struct cdaAli *cda);

/*** Return '+' or '-' depending on the strand that clone (.3/.5 pair) aligns on. */
char cdaCloneStrand(struct cdaAli *cda);

/*** Return '>' or '<' or ' ' depending whether cDNA is going same, opposite, or
 * unknown alignment as the chromosome strand. */
char cdaDirChar(struct cdaAli *cda, char chromStrand);

/*** Load in a string from CDA file. */
char *cdaLoadString(FILE *f);

/*** Read one block from cda file. */
void cdaReadBlock(FILE *f, struct cdaBlock *block);

/*** Call this to open file and verify signature, then call cdaLoadOne
 * which returns NULL at EOF. This file type is created by binGood.exe. */
FILE *cdaOpenVerify(char *fileName);

/*** Load one cdaAli from file.  Assumes file pointer is correctly positioned.
 * either by cdaOpenVerify or a previous cdaLoadOne.  Returns NULL at EOF. */
struct cdaAli *cdaLoadOne(FILE *f);

/*** Loop through blocks and figure out and fill in chromStart
 * and chromEnd. */
void cdaFixChromStartEnd(struct cdaAli *cda);

/*** Coalesce blocks separated by small amounts of noise. */
void cdaCoalesceBlocks(struct cdaAli *ca);

/*** Coalesce blocks as above, but don't update the score. */
void cdaCoalesceFast(struct cdaAli *ca);

void cdaShowAlignmentTrack(struct memGfx *mg, 
    int xOff, int yOff, int width, int height,  Color goodColor, Color badColor,
    int dnaSize, int dnaOffset, struct cdaAli *cda, char repeatChar);
/*** Draw alignment on a horizontal track of picture. */

/*** Reverse complement one cda. DnaStart is typically display window start. */
void cdaRcOne(struct cdaAli *cda, int dnaStart, int baseCount);

/*** Reverse complement cda list. */
void cdaRcList(struct cdaAli *cdaList, int dnaStart, int baseCount);

/*** Free a single cdaAli. */
void cdaFreeAli(struct cdaAli *ca);

/*** Free list of cdaAli. */
void cdaFreeAliList(struct cdaAli **pList);

struct cdaAli *cdaAliFromFfAli(struct ffAli *aliList, 
    DNA *needle, int needleSize, DNA *hay, int haySize, boolean isRc);
/*** Convert from ffAli to cdaAli format. */

/*** Write out a cdaList to a cda file. */
void cdaWrite(char *fileName, struct cdaAli *cdaList);

#endif /*** CDA_H */
