/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** ErrAbort.h - our error handler. 
 *
 * This maintains two stacks - a warning message printer
 * stack, and a "abort handler" stack.
 *
 * By default the warnings will go to stderr, and
 * aborts will exit the program.  You can push a
 * function on to the appropriate stack to change
 * this behavior.  The top function on the stack
 * gets called.
 *
 * Most functions in this library will call errAbort()
 * if they run out of memory.  
 */


/*** Abort function, with optional (printf formatted) error message. */
void errAbort(char *format, ...);

/*** Abort function, with optional (vprintf formatted) error message. */
void vaErrAbort(char *format, va_list args);

/*** Prints error message from UNIX errno first, then does errAbort. */
void errnoAbort(char *format, ...);

/*** Function that can abort. */
typedef void (*AbortHandler)();

/*** Set abort handler */
void pushAbortHandler(AbortHandler handler);

/*** Revert to old abort handler. */
void popAbortHandler();

/*** Abort without message. */
void noWarnAbort();

/*** Push abort handler that will invoke debugger. */
void pushDebugAbort();

/*** Call top of warning stack to issue warning. */
void vaWarn(char *format, va_list args);

/*** Issue a warning message. */
void warn(char *format, ...);

/*** Function that can warn. */
typedef void (*WarnHandler)(char *format, va_list args);

/*** Set warning handler */
void pushWarnHandler(WarnHandler handler);

/*** Revert to old warn handler. */
void popWarnHandler();

