/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** gdf - Intronerator Gene Description File. */

#ifndef GDF_H
#define GDF_H

#ifndef DNAUTIL_H
#include "dnautil.h"
#endif

/*** This stores data at each exon/intron boundary. */
struct gdfDataPoint
    {
    int start;
    };

/*** One structure of these for each gene (each isoform of each gene
 * actually. */
struct gdfGene
    {
    struct gdfGene *next;
    char *name;
    int dataCount;
    struct gdfDataPoint *dataPoints;
    char strand;
    UBYTE chromIx;
    };

/*** Return a new gene. */
struct gdfGene *newGdfGene(char *name, int nameSize, int exonCount, char strand, UBYTE chromIx);

/*** Free a gene. */
void gdfFreeGene(struct gdfGene *gene);

/*** Free a whole list of genes. */
void gdfFreeGeneList(struct gdfGene **pList);

/*** Read one entry from a Gdf file.  Assumes that the file pointer
 * is in the right place. */
struct gdfGene *gdfReadOneGene(FILE *f);

/*** Figure out first and last base in gene. */
void gdfGeneExtents(struct gdfGene *gene, long *pMin, long *pMax);

/*** Add offset to each point in gene */
void gdfOffsetGene(struct gdfGene *gene, int offset);

/*** Flip gene to other strand. Assumes dataPoints are already
 * moved into range from 0-size */
void gdfRcGene(struct gdfGene *gene, int size);

/*** Uppercase exons in DNA. */
void gdfUpcExons(struct gdfGene *gene, int geneOffset, DNA *dna, int dnaSize, int dnaOffset);


#endif /*** GDF_H */

