/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** keys.h - Stuff to manage a little key/value table and
 * evaluate expressions on it. */
#ifndef KEYS_H
#define KEYS_H

struct keyVal
    {
    char *key;
    char *val;
    };

/*** Get a new key value table. */
struct kvt *newKvt(int size);

/*** Free up key value table. */
void freeKvt(struct kvt **pKvt);

/*** Clear the keys table. */
void kvtClear(struct kvt *kvt);

/*** Add in new key. */
struct keyVal *kvtAdd(struct kvt *kvt, char *key, char *val);

/*** Search table for key.  Return key value, or NULL if
 * key not found. */
char *kvtLookup(struct kvt *kvt, char *key);

/*** Write all keys to file except the ones in hideList */
void kvtWriteAll(struct kvt *kvt, FILE *f, struct slName *hideList);

/*** Add in keys from text.  Text is in format:
 *     key val
 * for each line of text. Text gets many of it's
 * space characters and newlines replaced by 0's
 * and should persist until call to keysClear(). */
void kvtParseAdd(struct kvt *kvt, char *text);

/*** A handle on a parsed expression which can be
 * quickly evaluated.  */
    void *rootExp;       /*** Internally struct exp. */
    void *tokenList;     /*** Internally struct tok. */
    void *tokenList;     /*** Internally struct tok. */
    void *tokenList;     /*** Internally struct tok. */
    };

/*** Recursively evaluate expression. */
boolean keyExpEval(struct keyExp *exp, struct kvt *kvt);

/*** Parse text into key expression.  Squawk and die if it
 * fails. */
struct keyExp *keyExpParse(char *text);

/*** Get value of key in text. Return FALSE if key doesn't exist. */
boolean keyTextScan(char *text, char *key, char *valBuf, int valBufSize);

#endif /*** KEYS_H */

