/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** lineFile - stuff to rapidly read text files and parse them into
 * lines. */
#ifndef LINEFILE_H
#define LINEFILE_H

/*** Structure to handle fast, line oriented
 * fileIo. */
    struct lineFile *next;	/*** Might need to be on a list. */
    char *fileName;		/*** Name of file. */
    int fd;			/*** File handle. */
    int bufSize;		/*** Size of buffer. */
    int bufOffsetInFile;	/*** Offset in file of first buffer byte. */
    int bytesInBuf;		/*** Bytes read into buffer. */
    int lastReadSize;		/*** Size of last read. */
    int lineIx;			/*** Current line. */
    int lineStart;		/*** Offset of line in buffer. */
    int lineEnd;		/*** End of line in buffer. */
    bool zTerm;			/*** Replace '\n' with zero? */
    bool reuse;			/*** Set if reusing input. */
    char *buf;			/*** Buffer. */
    char *buf;			/*** Buffer. */
    char *buf;			/*** Buffer. */
    };

/*** Try and open up a lineFile. */
struct lineFile *lineFileMayOpen(char *fileName, bool zTerm);

/*** Open up a lineFile or die trying. */
struct lineFile *lineFileOpen(char *fileName, bool zTerm);

/*** Wrap a line file around an open'd file. */
struct lineFile *lineFileAttatch(char *fileName, bool zTerm, int fd);

/*** Wrap a line file around stdin. */
struct lineFile *lineFileStdin(bool zTerm);

/*** Close up a line file. */
void lineFileClose(struct lineFile **pLf);

/*** Fetch next line from file. */
boolean lineFileNext(struct lineFile *lf, char **retStart, int *retSize);

/*** Fetch next line from file.  Squawk and die if it's not there. */
void lineFileNeedNext(struct lineFile *lf, char **retStart, int *retSize);

/*** Reuse current line. */
void lineFileReuse(struct lineFile *lf);

/*** Current string in line file. */
#define lineFileString(lf) ((lf)->buf + (lf)->lineStart)

/*** Check line has right number of words. */
void lineFileExpectWords(struct lineFile *lf, int expecting, int got);

/*** Return next non-blank line that doesn't start with '#' chopped into words.
 * Returns FALSE at EOF.  Aborts on error. */
boolean lineFileNextRow(struct lineFile *lf, char *words[], int wordCount);

/*** Read in line chopped into fixed size word array. */
#define lineFileRow(lf, words) lineFileNextRow(lf, words, ArraySize(words))

/*** Return next non-blank line that doesn't start with '#' chopped into words. */
int lineFileChopNext(struct lineFile *lf, char *words[], int maxWords);

/*** Ease-of-usef macro for lineFileChopNext above. */
#define lineFileChop(lf, words) lineFileChopNext(lf, words, ArraySize(words))

/*** Make sure that words[wordIx] is an ascii integer, and return
 * binary representation of it. */
int lineFileNeedNum(struct lineFile *lf, char *words[], int wordIx);

/*** Skip a number of lines. */
void lineFileSkip(struct lineFile *lf, int lineCount);

#endif /*** LINEFILE_H */


