/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
#ifndef MEMALLOC_H
#define MEMALLOC_H

struct memHandler
    {
    struct memHandler *next;
    void * (*alloc)(size_t size);
    void (*free)(void *vpt);
    };

/*** Use newHandler for memory requests until matching popMemHandler.
 * Returns previous top of memory handler stack. */
struct memHandler *pushMemHandler(struct memHandler *newHandler);

/*** Removes top element from memHandler stack and returns it. */
struct memHandler *popMemHandler();

/*** Sets memHandler to the default. */
void setDefaultMemHandler();

/*** Push the careful (paranoid, conservative, checks everything)
 * memory handler  top of the memHandler stack and use it. */
void pushCarefulMemHandler(long maxAlloc);

/*** Walk through allocated memory and make sure that all cookies are
 * in place. Only walks through what's been done since 
 * pushCarefulMemHandler(). */
void carefulCheckHeap();

int carefulCountBlocksAllocated();
/*** How many memory items are allocated? (Since called
 * pushCarefulMemHandler(). */

#endif /*** MEMALLOC_H */

