/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** oldGff.h - This reads a *special* type of .GFF file.  
 * Some of the routines herein expect the gff file to include
 * both DNA and gene/intron/exon info.  They expect the
 * genes to be named by the group field. */
#ifndef OLDGFF_H
#define OLDGFF_H

/*** This is the structure that holds info on a gff file.
 */
struct gff
    {
    char fileName[256];
    FILE *file;
    long fileSize;
    char buf[256];
    int bufSize;
    int bytesInBuf;
    int readIx;
    int lineNumber;
    struct lm *memPool;
    char *dna;
    long dnaSize;
    char dnaName[128];
    struct gffGene *genes;
    };

/*** An intron or exon - just an offset into a DNA array. */
    struct gffSegment *next;	/*** This must be first field! */
    struct gffSegment *next;	/*** This must be first field! */
    struct gffSegment *next;	/*** This must be first field! */
    long start, end;
    int frame;
    };
typedef struct gffSegment GffIntron;
typedef struct gffSegment GffExon;

/*** At the end of a GFF file are a number of genes, each of which 
 * is a list of exons/introns. */
    struct gffGene *next;	/*** This must be first field! */
    struct gffGene *next;	/*** This must be first field! */
    struct gffGene *next;	/*** This must be first field! */
    char strand;	/*** + or - */
    char strand;	/*** + or - */
    char strand;	/*** + or - */
    char name[64];
    GffExon *exons;
    GffIntron *introns;
    DNA *dna;
    long dnaSize;
    };


/*** Initialize gff structure and open file for it. */
boolean gffOpen(struct gff *gff, char *fileName);

/*** Open up gff file and read everything in it. */
boolean gffOpenAndRead(struct gff *gff, char *fileName);

/*** Close down gff structure. */
void gffClose(struct gff *gff);

/*** Read all the DNA in a file. */
boolean gffReadDna(struct gff *gff);

/*** Find gene with given name.  Case sensitive. */
struct gffGene *gffFindGene(struct gff *gff, char *geneName);

/*** Find gene with given name.  Not case sensitive. */
struct gffGene *gffFindGeneIgnoreCase(struct gff *gff, char *geneName);

/*** Print summary info about file. */
void gffPrintInfo(struct gff *gff, FILE *out);

/*** Read all the gene (as opposed to base) info in file. */
boolean gffReadGenes(struct gff *gff);

/*** Make a duplicate of gene (with it's own DNA). gffFreeGene it when done. */
struct gffGene *gffDupeGene(struct gff *gff, struct gffGene *oldGene);

struct gffGene *gffDupeGeneAndSurrounds(struct gff *gff, struct gffGene *oldGene,
    int leftExtra, int rightExtra);
/*** Make a duplicate of gene with extra DNA around coding region. 
 * gffFreeGene it when done. */

/*** Find gene with given name.  Case sensitive. */
struct gffGene *gffGeneWithOwnDna(struct gff *gff, char *geneName);

/*** Free a gene returned with dupeGene or geneWithOwnDna. 
 * (You don't want to free the ones returned by findGene,
 * they are still owned by the gff.)
 */
void gffFreeGene(struct gffGene **pGene);

/*** Open gff file and read DNA sequence from it. */
struct dnaSeq *gffReadDnaSeq(char *fileName);

#endif /*** GFF_H */

