/*** psl.h was originally generated by the autoSql program, which also 
 * generated psl.c and psl.sql.  This header links the database and 
 * the RAM representation of objects.   Additional functions were
 * added later. */

#ifndef PSL_H
#define PSL_H

#ifndef LOCALMEM_H
#include "localmem.h"
#endif 

#ifndef LINEFILE_H
#include "linefile.h"
#endif

#ifndef FUZZYFIND_H
#include "fuzzyFind.h"
#endif

#ifndef DNASEQ_H
#include "dnaseq.h"
#endif

/*** Summary info about a patSpace alignment */
    struct psl *next;  /*** Next in singly linked list. */
    unsigned match;	/*** Number of bases that match that aren't repeats */
    unsigned misMatch;	/*** Number of bases that don't match */
    unsigned repMatch;	/*** Number of bases that match but are part of repeats */
    unsigned nCount;	/*** Number of 'N' bases */
    unsigned qNumInsert;	/*** Number of inserts in query */
    int qBaseInsert;	/*** Number of bases inserted in query */
    unsigned tNumInsert;	/*** Number of inserts in target */
    int tBaseInsert;	/*** Number of bases inserted in target */
    char strand[3];	/*** + or - for strand */
    char *qName;	/*** Query sequence name */
    unsigned qSize;	/*** Query sequence size */
    int qStart;	/*** Alignment start position in query */
    int qEnd;	/*** Alignment end position in query */
    char *tName;	/*** Target sequence name */
    unsigned tSize;	/*** Target sequence size */
    int tStart;	/*** Alignment start position in target */
    int tEnd;	/*** Alignment end position in target */
    unsigned blockCount;	/*** Number of blocks in alignment */
    unsigned *blockSizes;	/*** Size of each block */
    unsigned *qStarts;	/*** Start of each block in query. */
    unsigned *tStarts;	/*** Start of each block in target. */
    unsigned *tStarts;	/*** Start of each block in target. */
    unsigned *tStarts;	/*** Start of each block in target. */
    };

/*** Load a psl from row fetched with select * from psl
 * from database.  Dispose of this with pslFree(). */
struct psl *pslLoad(char **row);

/*** Create a psl out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new psl */
struct psl *pslCommaIn(char **pS, struct psl *ret);

/*** Free a single dynamically allocated psl such as created
 * with pslLoad(). */
void pslFree(struct psl **pEl);

/*** Free a list of dynamically allocated psl's */
void pslFreeList(struct psl **pList);

/*** Print out psl.  Separate fields with sep. Follow last field with lastSep. */
void pslOutput(struct psl *el, FILE *f, char sep, char lastSep);

/*** Print out psl as a line in a tab-separated file. */
#define pslTabOut(el,f) pslOutput(el,f,'\t','\n')

/*** Print out psl as a comma separated list including final comma. */
#define pslCommaOut(el,f) pslOutput(el,f,',',',')

/*** ----- end autoSql generated part --------------- */

/*** Load all psl's in file. */
struct psl *pslLoadAll(char *fileName);

/*** Read next line from file and convert it to psl.  Return
 * NULL at eof. */
struct psl *pslNext(struct lineFile *lf);

/*** Load row into local memory psl. */
struct psl *pslLoadLm(char **row, struct lm *lm);

/*** Write head of psl. */
void pslWriteHead(FILE *f);

/*** Write head of pslx (extended psl). */
void pslxWriteHead(FILE *f, enum gfType qType, enum gfType tType);

/*** Write a psl file from list. */
void pslWriteAll(struct psl *pslList, char *fileName, boolean writeHeader);

/*** Read header part of psl and make sure it's right. 
 * Return line file handle to it. */
struct lineFile *pslFileOpen(char *fileName);

void pslxFileOpen(char *fileName, enum gfType *retQueryType, 
	enum gfType *retTargetType, struct lineFile **retLf);
/*** Read header part of psl and make sure it's right.  Return
 * sequence types and file handle. */

/*** Compare to sort based on query. */
int pslCmpQuery(const void *va, const void *vb);

/*** Compare to sort based on target. */
int pslCmpTarget(const void *va, const void *vb);

/*** Calculate badness in parts per thousand. */
int pslCalcMilliBad(struct psl *psl, boolean isMrna);

struct ffAli *pslToFfAli(struct psl *psl, struct dnaSeq *query, struct dnaSeq *target,
	int targetOffset);
/*** Convert from psl to ffAli format. */

/*** Convert from psl to ffAli format.  In some cases you can pass NULL
 * for needle and haystack - depending what the post-processing is going
 * to be. */
struct ffAli *pslToFakeFfAli(struct psl *psl, DNA *needle, DNA *haystack);

struct psl *pslFromFakeFfAli(struct ffAli *ff, 
	DNA *needle, DNA *haystack, char strand,
	char *qName, int qSize, char *tName, int tSize);
/*** This will create a basic psl structure from a sorted series of ffAli
 * blocks.  The fields that would need actual sequence to be filled in
 * are left zero however - fields including match, repMatch, mismatch. */

/*** Translate psl strand + or - to orientation +1 or -1 */
int pslOrientation(struct psl *psl);

/*** Find the length of "tails" (rather than extensions) implied by psl. */
void pslTailSizes(struct psl *psl, int *retStartTail, int *retEndTail);

/*** Swap around things in psl so it works as if the alignment
 * was done on the reverse strand of the target. */
void pslRcBoth(struct psl *psl);

/*** Add offset to target positions in psl. */
void pslTargetOffset(struct psl *psl, int offset);

/*** Dump most of PSL to file - for debugging. */
void pslDump(struct psl *psl, FILE *f);

#endif /*** PSL_H */

