/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** ShaRes.c - implementation of shared resources */
#include "common.h"
#include "shaRes.h"

/*** Free a node.  (Don't look at link count, just do it.) */
static void shaFreeNode(struct shaResNode *node)
{
struct shaResNode *next = node->next;
struct shaResNode *prev = node->prev;
struct shaResList *list = node->list;

if (next != NULL)
    next->prev = prev;
if (prev != NULL)
    prev->next = next;
else
    list->head = next;
list->freeData(node->data);
freeMem(node->name);
freeMem(node);
}        

/*** Decrement link count and free if down to zero. */
void shaUnlink(struct shaResNode *node)
{
if (--node->links <= 0)
    shaFreeNode(node);
}

/*** Increment link count. */
void shaLink(struct shaResNode *node)
{
++node->links;
}

/*** Create a new node with link count of one. */
struct shaResNode *shaNewNode(struct shaResList *list, char *name, void *data)
{
struct shaResNode *nn = needMem(sizeof(*nn));
struct shaResNode *head = list->head;

/*** Store the goods and what list we're on, and start with one link. */
nn->list = list;
nn->data = data;
nn->links = 1;
nn->name = cloneString(name);

/*** Put us at the front of the list. */
nn->next = head;
nn->prev = NULL;
if (head != NULL)
    head->prev = nn;
list->head = nn;
return nn;
}

/*** Free every node on list. */
void shaCleanup(struct shaResList *list)
{
struct shaResNode *node, *next;

for (node = list->head;node != NULL;node = next)
    {
    next = node->next;
    shaFreeNode(node);
    }
list->head = NULL;
}

/*** Start up resource list. */
void shaInit(struct shaResList *list, void (*freeData)(void *pData))
{
list->head = NULL;
list->freeData = freeData;
}
