/*******************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/*** Stuff for processing comma separated lists - a little long so
 * in a separate module from jksql.c though interface is still
 # in jksql.c. */

#include "common.h"
#include "sqlNum.h"
#include "sqlList.h"

/*** Convert comma separated list of numbers to an array.  Pass in 
 * array an max size of array. */
int sqlByteArray(char *s, signed char *array, int arraySize)
{
unsigned count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == arraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = sqlSigned(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlByteStaticArray(char *s, signed char **retArray, int *retSize)
{
static signed char *array = NULL;
static unsigned alloc = 0;
unsigned count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = sqlSigned(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlByteDynamicArray(char *s, signed char **retArray, int *retSize)
{
signed char *sArray, *dArray = NULL;
unsigned size;

sqlByteStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of numbers to an array.  Pass in 
 * array an max size of array. */
int sqlUbyteArray(char *s, unsigned char *array, int arraySize)
{
unsigned count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == arraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = sqlUnsigned(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlUbyteStaticArray(char *s, unsigned char **retArray, int *retSize)
{
static unsigned char *array = NULL;
static unsigned alloc = 0;
unsigned count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = sqlUnsigned(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlUbyteDynamicArray(char *s, unsigned char **retArray, int *retSize)
{
unsigned char *sArray, *dArray = NULL;
unsigned size;

sqlUbyteStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of numbers to an array.  Pass in 
 * array an max size of array. */
int sqlShortArray(char *s, short *array, int arraySize)
{
unsigned count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == arraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = sqlSigned(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlShortStaticArray(char *s, short **retArray, int *retSize)
{
static short *array = NULL;
static unsigned alloc = 0;
unsigned count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = sqlSigned(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlShortDynamicArray(char *s, short **retArray, int *retSize)
{
short *sArray, *dArray = NULL;
unsigned size;

sqlShortStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of numbers to an array.  Pass in 
 * array an max size of array. */
int sqlUshortArray(char *s, unsigned short *array, int arraySize)
{
unsigned count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == arraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = sqlUnsigned(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlUshortStaticArray(char *s, unsigned short **retArray, int *retSize)
{
static unsigned short *array = NULL;
static unsigned alloc = 0;
unsigned count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = sqlUnsigned(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlUshortDynamicArray(char *s, unsigned short **retArray, int *retSize)
{
unsigned short *sArray, *dArray = NULL;
unsigned size;

sqlUshortStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of floating point numbers to an array.  
 * Pass in array and max size of array. */
int sqlFloatArray(char *s, float *array, int maxArraySize)
{
unsigned count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == maxArraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = atof(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlFloatStaticArray(char *s, float **retArray, int *retSize)
{
static float *array = NULL;
static unsigned alloc = 0;
unsigned count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = atof(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlFloatDynamicArray(char *s, float **retArray, int *retSize)
{
float *sArray, *dArray = NULL;
unsigned size;

sqlFloatStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of numbers to an array.  Pass in 
 * array and max size of array. */
int sqlUnsignedArray(char *s, unsigned *array, int arraySize)
{
unsigned count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == arraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = sqlUnsigned(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlUnsignedStaticArray(char *s, unsigned **retArray, int *retSize)
{
static unsigned *array = NULL;
static unsigned alloc = 0;
unsigned count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = sqlUnsigned(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlUnsignedDynamicArray(char *s, unsigned **retArray, int *retSize)
{
unsigned *sArray, *dArray = NULL;
unsigned size;

sqlUnsignedStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of numbers to an array.  Pass in 
 * array an max size of array. */
int sqlSignedArray(char *s, int *array, int arraySize)
{
int count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == arraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = sqlSigned(s);
    s = e;
    }
return count;
}

/*** Convert comma separated list of numbers to an array which will be
 * overwritten next call to this function, but need not be freed. */
void sqlSignedStaticArray(char *s, int **retArray, int *retSize)
{
static int *array = NULL;
static int alloc = 0;
int count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = sqlSigned(s);
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of numbers to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlSignedDynamicArray(char *s, int **retArray, int *retSize)
{
int *sArray, *dArray = NULL;
int size;

sqlSignedStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/***-------------------------*/

/*** Convert comma separated list of strings to an array.  Pass in 
 * array and max size of array.  Returns actual size*/
int sqlStringArray(char *s, char **array, int maxArraySize)
{
int count = 0;
for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0 || count == maxArraySize)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    array[count++] = s;
    s = e;
    }
return count;
}

/*** Convert comma separated list of strings to an array which will be
 * overwritten next call to this function or to sqlUnsignedDynamicArray,
 * but need not be freed. */
void sqlStringStaticArray(char *s, char  ***retArray, int *retSize)
{
static char **array = NULL;
static int alloc = 0;
int count = 0;

for (;;)
    {
    char *e;
    if (s == NULL || s[0] == 0)
	break;
    e = strchr(s, ',');
    if (e != NULL)
	*e++ = 0;
    if (count >= alloc)
	{
	if (alloc == 0)
	    alloc = 64;
	else
	    alloc <<= 1;
	ExpandArray(array, count, alloc);
	}
    array[count++] = s;
    s = e;
    }
*retSize = count;
*retArray = array;
}

/*** Convert comma separated list of strings to an dynamically allocated
 * array, which should be freeMem()'d when done. */
void sqlStringDynamicArray(char *s, char ***retArray, int *retSize)
{
char **sArray, **dArray = NULL;
int size;
int i;

s = cloneString(s);
sqlStringStaticArray(s, &sArray, &size);
if (size > 0)
    {
    AllocArray(dArray,size);
    CopyArray(sArray, dArray, size);
    }
*retArray = dArray;
*retSize = size;
}

/*** Free up a dynamic array (ends up freeing array and first string on it.) */
void sqlStringFreeDynamicArray(char ***pArray)
{
char **array;
if ((array = *pArray) != NULL)
    {
    freeMem(array[0]);
    freez(pArray);
    }
}

/*** Return signed number at *pS.  Advance *pS past comma at end */
int sqlUnsignedComma(char **pS)
{
char *s = *pS;
char *e = strchr(s, ',');
unsigned ret;

*e++ = 0;
*pS = e;
ret = sqlUnsigned(s);
return ret;
}

/*** Return signed number at *pS.  Advance *pS past comma at end */
int sqlSignedComma(char **pS)
{
char *s = *pS;
char *e = strchr(s, ',');
int ret;

*e++ = 0;
*pS = e;
ret = sqlSigned(s);
return ret;
}

/*** Return end of string. */
static char *findStringEnd(char *start, char endC)
{
char c;
char *s = start;

for (;;)
    {
    c = *s;
    if (c == endC)
	return s;
    else if (c == 0)
	errAbort("Unterminated string");
    ++s;
    }
}

/*** Return string at *pS.  (Either quoted or not.)  Advance *pS. */
static char *sqlGetOptQuoteString(char **pS)
{
char *s = *pS;
char *e;
char c = *s;

if (c  == '"' || c == '\'')
    {
    s += 1;
    e = findStringEnd(s, c);
    *e++ = 0;
    if (*e++ != ',')
	errAbort("Expecting comma after string");
    }
else
    {
    e = strchr(s, ',');
    *e++ = 0;
    }
*pS = e;
return s;
}

/*** Return string at *pS.  (Either quoted or not.)  Advance *pS. */
char *sqlStringComma(char **pS)
{
return cloneString(sqlGetOptQuoteString(pS));
}

/*** Copy string at *pS to buf.  Advance *pS. */
void sqlFixedStringComma(char **pS, char *buf, int bufSize)
{
strncpy(buf, sqlGetOptQuoteString(pS), bufSize);
}

/*** Make sure next character is 'c'.  Return past next char */
char *sqlEatChar(char *s, char c)
{
if (*s++ != c)
    errAbort("Expecting %c got %c (%d) in database", c, s[-1], s[-1]);
return s;
}

