/** subChar - Substitute one character for another throughout a file.. */
#include "common.h"
#include "obscure.h"

/** Explain usage and exit. */
void usage()
{
errAbort(
  "subChar - Substitute one character for another throughout a file.\n"
  "usage:\n"
  "   subChar oldChar newChar file(s)\n"
  "oldChar and newChar can either be single letter literal characters,\n"
  "or two digit hexadecimal ascii codes");
}

/** Return hex val of s. */
int unhex(char *s)
{
int acc = 0;
char c;

tolowers(s);
while ((c = *s++) != 0)
    {
    acc <<= 4;
    if (c >= '0' && c <= '9')
       acc += c - '0';
    else if (c >= 'a' && c <= 'f')
       acc += c + 10 - 'a';
    else
       errAbort("Expecting hexadecimal character got %s", s);
    }
return acc;
}

/** Return hexadecimal or literal decoding of string into char. */
char charVal(char *s)
{
int len = strlen(s);
if (len == 1)
   return s[0];
else if (len == 2)
   return unhex(s);
else
   usage();
}

/** Substitute new for old throughout file. */
void subCharsInFile(char old, char new, char *fileName)
{
int subCount = 0;
int fileSize;
int i;
char *buf;

readInGulp(fileName, &buf, &fileSize);
for (i=0; i<fileSize; ++i)
    {
    if (buf[i] == old)
       {
       buf[i] = new;
       ++subCount;
       }
    }
if (subCount > 0)
    {
    FILE *f = mustOpen(fileName, "w");
    printf("Substituting %x for %x %d times in %s\n",
    	new, old, subCount, fileName);
    mustWrite(f, buf, fileSize);
    fclose(f);
    }
freez(&buf);
}

/** subChar - Substitute one character for another throughout a file.. */
void subChars(char *oldChar, char *newChar, int fileCount, char *fileNames[])
{
char old = charVal(oldChar), new = charVal(newChar);
int i;
char *fileName;

for (i=0; i<fileCount; ++i)
    {
    fileName = fileNames[i];
    subCharsInFile(old, new, fileName);
    }
}

/** Process command line. */
int main(int argc, char *argv[])
{
if (argc < 4)
    usage();
subChars(argv[1], argv[2], argc-3, argv+3);
return 0;
}
